/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.cocoon.spring.configurator.impl;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanDefinitionStoreException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanFactoryAware;
import org.springframework.beans.factory.HierarchicalBeanFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.ListableBeanFactory;

/**
 * This is a map implementation collecting all beans of a specific type (class)
 * from a spring bean factory.
 * The beans are available through their configured bean id.
 *
 * @version $Id: BeanMap.java 509512 2007-02-20 11:24:10Z cziegeler $
 * @since 1.0.1
 */
public class BeanMap
    implements Map, BeanFactoryAware, InitializingBean {

    /** The real map. */
    protected Map beanMap = new HashMap();

    /** The bean names. */
    protected Set beanNames = new HashSet();

    /** Is the map initialized? */
    protected boolean initialized = false;

    /** The bean factory. */
    protected ListableBeanFactory beanFactory;

    /** The class for all beans in this map. */
    protected Class beanClass;

    protected boolean stripPrefix = true;
    protected boolean checkParent = true;

    protected void load() {
        final String prefix1 = this.beanClass.getName() + '.';
        final String prefix2 = this.beanClass.getName() + '/';
        final Iterator i = this.beanNames.iterator();
        while ( i.hasNext() ) {
            final String beanName = (String)i.next();
            String key = beanName;
            if ( this.stripPrefix && (beanName.startsWith(prefix1) || beanName.startsWith(prefix2)) ) {
                key = key.substring(prefix1.length());
            }
            this.beanMap.put(key, this.beanFactory.getBean(beanName));
        }
    }

    protected void checkInit() {
        if ( !this.initialized ) {
            synchronized (this) {
                if ( !this.initialized ) {
                    this.initialized = true;
                    this.load();
                }
            }
        }
    }

    /**
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    public void afterPropertiesSet() throws Exception {
        this.beanNames.clear();
        this.getBeanNames(this.beanFactory, this.beanNames);
    }

    /** 
     * Initialize this bean.
     */
    protected void getBeanNames(ListableBeanFactory factory, Set names) {
        // check parent first
        if ( this.checkParent ) {
            if ( factory instanceof HierarchicalBeanFactory ) {
                if ( ((HierarchicalBeanFactory)factory).getParentBeanFactory() != null ) {
                    this.getBeanNames((ListableBeanFactory)((HierarchicalBeanFactory)factory).getParentBeanFactory(), names);
                }
            }
        }
        // get all bean names for our class
        final String[] beanNames = factory.getBeanNamesForType(this.beanClass);
        for (int i = 0; i < beanNames.length; i++) {
            names.add(beanNames[i]);
        }
    }

    /**
     * @see org.springframework.beans.factory.BeanFactoryAware#setBeanFactory(org.springframework.beans.factory.BeanFactory)
     */
    public void setBeanFactory(BeanFactory factory) throws BeansException {
        if ( !(factory instanceof ListableBeanFactory) ) {
            throw new BeanDefinitionStoreException("BeanFactory must be listable.");
        }
        this.beanFactory = (ListableBeanFactory)factory;
    }

    public void setStripPrefix(boolean stripPrefix) {
        this.stripPrefix = stripPrefix;
    }

    public void setCheckParent(boolean checkParent) {
        this.checkParent = checkParent;
    }

    public void setType(Class typeClass) {
        this.beanClass = typeClass;
    }

    /**
     * @see java.util.Map#clear()
     */
    public void clear() {
        // no need to call checkInit as we clear the map!
        this.initialized = true;
        this.beanMap.clear();
    }

    /**
     * @see java.util.Map#containsKey(java.lang.Object)
     */
    public boolean containsKey(Object key) {
        this.checkInit();
        return this.beanMap.containsKey(key);
    }

    /**
     * @see java.util.Map#containsValue(java.lang.Object)
     */
    public boolean containsValue(Object value) {
        this.checkInit();
        return this.beanMap.containsValue(value);
    }

    /**
     * @see java.util.Map#entrySet()
     */
    public Set entrySet() {
        this.checkInit();
        return this.beanMap.entrySet();
    }

    /**
     * @see java.util.Map#get(java.lang.Object)
     */
    public Object get(Object key) {
        this.checkInit();
        return this.beanMap.get(key);
    }

    /**
     * @see java.util.Map#isEmpty()
     */
    public boolean isEmpty() {
        this.checkInit();
        return this.beanMap.isEmpty();
    }

    /**
     * @see java.util.Map#keySet()
     */
    public Set keySet() {
        this.checkInit();
        return this.beanMap.keySet();
    }

    /**
     * @see java.util.Map#put(java.lang.Object, java.lang.Object)
     */
    public Object put(Object key, Object value) {
        this.checkInit();
        return this.beanMap.put(key, value);
    }

    /**
     * @see java.util.Map#putAll(java.util.Map)
     */
    public void putAll(Map t) {
        this.checkInit();
        this.beanMap.putAll(t);
    }

    /**
     * @see java.util.Map#remove(java.lang.Object)
     */
    public Object remove(Object key) {
        this.checkInit();
        return this.beanMap.remove(key);
    }

    /**
     * @see java.util.Map#size()
     */
    public int size() {
        this.checkInit();
        return this.beanMap.size();
    }

    /**
     * @see java.util.Map#values()
     */
    public Collection values() {
        this.checkInit();
        return this.beanMap.values();
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        this.checkInit();
        return this.beanMap.equals(obj);
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        this.checkInit();
        return this.beanMap.hashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        this.checkInit();
        return this.beanMap.toString();
    }
}
